/* Тестовый пример иллюстрирует применение режима гаммирования.
   Внимание! Используются не экспортируемые функции.

   test-bckey02.c
*/
 #include <stdio.h>
 #include <stdlib.h>
 #include <ak_bckey.h>
 #include <ak_tools.h>

/*
   инвертированные тестовые значения взяты из
   https://github.com/gost-engine/engine/blob/master/test_grasshopper.c

   запуск
     test-bckey02 проводит тестирование для оригинальной реализации
     test-bckey02 1 проводит тестирование для реализации, совместимой с openssl */

 int main( int argc, char *argv[] )
{
  int result = EXIT_FAILURE;

 /* устанавливаем флаг совместимости с openssl: 0 - нет совместимости, 1 - есть */
  int i, j, oc = 0;
  ak_uint8 buf[128], *ptr;
  struct bckey bkey, mkey;

 /* значение секретного ключа согласно ГОСТ Р 34.12-2015 для алгоритма Кузнечик */
  ak_uint8 key[32] = {
    0xef, 0xcd, 0xab, 0x89, 0x67, 0x45, 0x23, 0x01, 0x10, 0x32, 0x54, 0x76, 0x98, 0xba, 0xdc, 0xfe,
    0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88 };

 /* последовательность символов для передачи в командную строку
    8899aabbccddeeff0011223344556677fedcba98765432100123456789abcdef */
  ak_uint8 openssl_key[32] = {
    0x88,0x99,0xaa,0xbb,0xcc,0xdd,0xee,0xff,0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,
    0xfe,0xdc,0xba,0x98,0x76,0x54,0x32,0x10,0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef };

  ak_uint8 magma_key[32] = { /* тестовый ключ из ГОСТ Р 34.12-2015, приложение А.2 */
     0xff, 0xfe, 0xfd, 0xfc, 0xfb, 0xfa, 0xf9, 0xf8, 0xf7, 0xf6, 0xf5, 0xf4, 0xf3, 0xf2, 0xf1, 0xf0,
     0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff
  };
 /* последовательность символов для передачи в командную строку
    ffeeddccbbaa99887766554433221100f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff */
  ak_uint8 openssl_magma_key[32] = { /* тестовый ключ из ГОСТ Р 34.12-2015, приложение А.2 (развернутый) */
     0xff,0xee,0xdd,0xcc,0xbb,0xaa,0x99,0x88,0x77,0x66,0x55,0x44,0x33,0x22,0x11,0x00,
     0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xff
  };

 /* открытый текст из ГОСТ Р 34.12-2015, приложение А.1, подлежащий зашифрованию */
  ak_uint8 in[67] = {
    0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00, 0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11,
    0x0a, 0xff, 0xee, 0xcc, 0xbb, 0xaa, 0x99, 0x88, 0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
    0x00, 0x0a, 0xff, 0xee, 0xcc, 0xbb, 0xaa, 0x99, 0x88, 0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11,
    0x11, 0x00, 0x0a, 0xff, 0xee, 0xcc, 0xbb, 0xaa, 0x99, 0x88, 0x77, 0x66, 0x55, 0x44, 0x33, 0x22,
    0x01, 0x02, 0x03
  };

  ak_uint8 openssl_in[67] = {
    0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x00,0xFF,0xEE,0xDD,0xCC,0xBB,0xAA,0x99,0x88,
    0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xEE,0xFF,0x0A,
    0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xEE,0xFF,0x0A,0x00,
    0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xEE,0xFF,0x0A,0x00,0x11,
    0x01,0x02,0x03
  };

 /* открытый текст из ГОСТ Р 34.13-2015, приложение А.2 */
  ak_uint8 magma_in[35] = {
     0x59, 0x0a, 0x13, 0x3c, 0x6b, 0xf0, 0xde, 0x92,
     0x20, 0x9d, 0x18, 0xf8, 0x04, 0xc7, 0x54, 0xdb,
     0x4c, 0x02, 0xa8, 0x67, 0x2e, 0xfb, 0x98, 0x4a,
     0x41, 0x7e, 0xb5, 0x17, 0x9b, 0x40, 0x12, 0x89,
     0x00, 0x00, 0x00
 };

  ak_uint8 openssl_magma_in[35] = {
    0x92, 0xde, 0xf0, 0x6b, 0x3c, 0x13, 0x0a, 0x59,
    0xdb, 0x54, 0xc7, 0x04, 0xf8, 0x18, 0x9d, 0x20,
    0x4a, 0x98, 0xfb, 0x2e, 0x67, 0xa8, 0x02, 0x4c,
    0x89, 0x12, 0x40, 0x9b, 0x17, 0xb5, 0x7e, 0x41,
    0x00, 0x00, 0x00
 };

 /* инициализационный вектор для режима гаммирования (счетчика) */
  ak_uint8 ivctr[8] = { 0xf0, 0xce, 0xab, 0x90, 0x78, 0x56, 0x34, 0x12 };
  ak_uint8 magma_ivctr[4] = { 0x78, 0x56, 0x34, 0x12 };

 /* последовательность символов для передачи в командную строку
    1234567890abcef00000000000000000 */
  ak_uint8 openssl_ivctr[8] = { 0x12,0x34,0x56,0x78,0x90,0xab,0xce,0xf0 };
 /* последовательность символов для передачи в командную строку
    1234567800000000 */
  ak_uint8 openssl_magma_ivctr[4] = { 0x12,0x34,0x56,0x78 };

 /* зашифрованный блок из ГОСТ Р 34.12-2015 */
  ak_uint8 outctr[67] = {
    0xb8, 0xa1, 0xbd, 0x40, 0xa2, 0x5f, 0x7b, 0xd5, 0xdb, 0xd1, 0x0e, 0xc1, 0xbe, 0xd8, 0x95, 0xf1,
    0xe4, 0xde, 0x45, 0x3c, 0xb3, 0xe4, 0x3c, 0xf3, 0x5d, 0x3e, 0xa1, 0xf6, 0x33, 0xe7, 0xee, 0x85,
    0xa5, 0xa3, 0x64, 0x35, 0xf1, 0x77, 0xe8, 0xd5, 0xd3, 0x6e, 0x35, 0xe6, 0x8b, 0xe8, 0xea, 0xa5,
    0x73, 0xba, 0xbd, 0x20, 0x58, 0xd1, 0xc6, 0xd1, 0xb6, 0xba, 0x0c, 0xf2, 0xb1, 0xfa, 0x91, 0xcb,
    0xc7, 0x28, 0x6c
  };
  ak_uint8 openssl_outctr[67] = {
    0xf1,0x95,0xd8,0xbe,0xc1,0x0e,0xd1,0xdb,0xd5,0x7b,0x5f,0xa2,0x40,0xbd,0xa1,0xb8,
    0x85,0xee,0xe7,0x33,0xf6,0xa1,0x3e,0x5d,0xf3,0x3c,0xe4,0xb3,0x3c,0x45,0xde,0xe4,
    0xa5,0xea,0xe8,0x8b,0xe6,0x35,0x6e,0xd3,0xd5,0xe8,0x77,0xf1,0x35,0x64,0xa3,0xa5,
    0xcb,0x91,0xfa,0xb1,0xf2,0x0c,0xba,0xb6,0xd1,0xc6,0xd1,0x58,0x20,0xbd,0xba,0x73,
    0x6e,0x28,0xc5
  };

  ak_uint8 magma_outctr[35] = {
    0x3c, 0xb9, 0xb7, 0x97, 0x0c, 0x11, 0x98, 0x4e,
    0x69, 0x5d, 0xe8, 0xd6, 0x93, 0x0d, 0x25, 0x3e,
    0xef, 0xdb, 0xb2, 0x07, 0x88, 0x86, 0x6d, 0x13,
    0x2d, 0xa1, 0x52, 0xab, 0x80, 0xb6, 0x8e, 0x56,
    0x05, 0x43, 0x03
  };
  ak_uint8 openssl_magma_outctr[35] = {
    0x4e, 0x98, 0x11, 0x0c, 0x97, 0xb7, 0xb9, 0x3c,
    0x3e, 0x25, 0x0d, 0x93, 0xd6, 0xe8, 0x5d, 0x69,
    0x13, 0x6d, 0x86, 0x88, 0x07, 0xb2, 0xdb, 0xef,
    0x56, 0x8e, 0xb6, 0x80, 0xab, 0x52, 0xa1, 0x2d,
    0x03, 0x43, 0x05
  };

  FILE *fp = NULL;

 /* сохраняем данные в файл, чтобы можно было потом проверить алгоритм Кузнечик с помощью библиотеки openssl */
  fp = fopen( "in.dat", "wb" ); fwrite( openssl_in, 67, 1, fp ); fclose( fp );
  fp = fopen( "in.dat.encx", "wb" ); fwrite( openssl_outctr, 67, 1, fp ); fclose( fp );
 /* сохраняем данные в файл, чтобы можно было потом проверить алгоритм Магма с помощью библиотеки openssl */
  fp = fopen( "mag.dat", "wb" ); fwrite( openssl_magma_in, 35, 1, fp ); fclose( fp );
  fp = fopen( "mag.dat.encx", "wb" ); fwrite( openssl_magma_outctr, 35, 1, fp ); fclose( fp );

 /* теперь можно проверить эквивалентность шифрования выполнив следующую команду,
    параметры которой указаны в настоящем файле выше

    openssl enc -e -grasshopper-ctr -in in.dat -out in.dat.enc -iv <значение iv> -K <значение ключа>
   или

    openssl enc -e -magma-ctr -in mag.dat -out mag.dat.enc -iv <значение iv> -K <значение ключа>      */


 /* передаем в программу значение флага совместимости */
  if( argc > 1 ) oc = atoi( argv[1] );
  if( oc != 1 ) oc = 0;

 /* инициализируем библиотеку */
  if( !ak_libakrypt_create( ak_function_log_stderr )) return ak_libakrypt_destroy();

 /* устанавливаем нужный вариант совместимости и пересчитываем внутренние таблицы */
  ak_libakrypt_set_option( "openssl_compability", oc );
  ak_bckey_context_kuznechik_init_gost_tables();
  oc ? printf("openssl_compability is ON\n") : printf("openssl_compability is OFF\n");

 /* создаем секретный ключ алгоритма Кузнечик */
  if( ak_bckey_context_create_kuznechik( &bkey ) != ak_error_ok )
    return ak_libakrypt_destroy();
 /* создаем секретный ключ алгоритма Магма */
  if( ak_bckey_context_create_magma( &mkey ) != ak_error_ok )
    return ak_libakrypt_destroy();

 /* устанавливаем секретный ключ */
  ak_bckey_context_set_key( &bkey, oc ? openssl_key : key, sizeof( key ));
 /* устанавливаем секретный ключ */
  ak_bckey_context_set_key( &mkey, oc ? openssl_magma_key : magma_key, sizeof( magma_key ));

 /* тестируем алгоритм Кузнечик */
  printf("kuznechik\n");

 /* зашифровываем и расшифровываем всего четыре блока данных в режиме гаммирования */
  ak_bckey_context_ctr( &bkey, oc ? openssl_in : in, buf, sizeof( in ),
                                                                   oc ? openssl_ivctr : ivctr, 8 );
  printf("encrypted:\n");
  for( i = 0; i < 4; i++ ) {
    for( j = 0; j < 16; j++ ) printf(" %02x", buf[i*16+j] );
    printf("\n");
  }
  printf(" %02x %02x %02x\n", buf[64], buf[65], buf[66] );

  ptr = oc ? openssl_outctr : outctr;
  printf("\nexpected:\n");
  for( i = 0; i < 4; i++ ) {
    for( j = 0; j < 16; j++ ) printf(" %02x", ptr[i*16+j] );
    printf("\n");
  }
  printf(" %02x %02x %02x\n", ptr[64], ptr[65], ptr[66] );

  if( ak_ptr_is_equal( buf, ptr, 67 )) {
    printf("Ok\n\n");
    result = EXIT_SUCCESS;
  }  else printf("Wrong\n\n");


 /* тестируем алгоритм Магма */
  printf("magma\n");

 /* зашифровываем и расшифровываем всего четыре блока данных в режиме гаммирования */
  ak_bckey_context_ctr( &mkey, oc ? openssl_magma_in : magma_in, buf, sizeof( magma_in ),
                                                 oc ? openssl_magma_ivctr : magma_ivctr, 4 );
  printf("encrypted:\n");
  for( i = 0; i < 4; i++ ) {
    for( j = 0; j < 8; j++ ) printf(" %02x", buf[i*8+j] );
    printf("\n");
  }
  printf(" %02x %02x %02x\n", buf[32], buf[33], buf[34] );

  ptr = oc ? openssl_magma_outctr : magma_outctr;
  printf("\nexpected:\n");
  for( i = 0; i < 4; i++ ) {
    for( j = 0; j < 8; j++ ) printf(" %02x", ptr[i*8+j] );
    printf("\n");
  }
  printf(" %02x %02x %02x\n", ptr[32], ptr[33], ptr[34] );

  // не сходятся последние байты для openssl compability (Магма)

  if( ak_ptr_is_equal( buf, ptr, 35 )) {
    printf("Ok\n");
    if( result != EXIT_FAILURE ) result = EXIT_SUCCESS;
  }  else printf("Wrong\n");

 /* уничтожаем ключи и выходим */
  ak_bckey_context_destroy( &mkey );
  ak_bckey_context_destroy( &bkey );
  ak_libakrypt_destroy();
 return result;
}
